@@ -1,9 +1,9 @@
 #
-# Copyright (C) 2005-2006 Timothy D. Morgan
+# Copyright (C) 2005-2006,2011 Timothy D. Morgan
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation version 2 of the
+# as published by the Free Software Foundation version 3 of the
 # License.
 #
 # This program is distributed in the hope that it will be useful,
@@ -12,7 +12,7 @@
 # GNU General Public License for more details.
 #
 # vi:set tabsize=4:
-# $Id: grokevt.py 88 2006-12-11 03:09:47Z tim $
+# $Id: grokevt.py 118 2011-06-20 16:06:43Z tim $
 
 import sys
 import os
@@ -20,7 +20,7 @@ import time
 import types
 import re
 import struct
-import anydbm
+import dbm
 
 
 ################################################################################
@@ -47,9 +47,9 @@ output_encoding = 'utf-8'
 
 
 # Log format constants
-header_log_magic = "\x4c\x66\x4c\x65"
-header_version = "\x01\x00\x00\x00\x01\x00\x00\x00"
-cursor_magic ="\x11\x11\x11\x11\x22\x22\x22\x22\x33\x33\x33\x33\x44\x44\x44\x44"
+header_log_magic = b"\x4c\x66\x4c\x65"
+header_version = b"\x01\x00\x00\x00\x01\x00\x00\x00"
+cursor_magic = b"\x11\x11\x11\x11\x22\x22\x22\x22\x33\x33\x33\x33\x44\x44\x44\x44"
 
 header_size = 0x30
 cursor_size = 0x28
@@ -62,16 +62,20 @@ min_record_size = min(header_size, cursor_size, log_fixed_size)
 
 # Reference:
 #  http://blogs.msdn.com/oldnewthing/archive/2004/03/15/89753.aspx
+#
+# XXX: Need to replace this with calls to winsec library once winsec has a 
+#      better memory management interface
 def binSIDtoASCII(sid_str):
-    auth = ((ord(sid_str[2])<<40)
-            | (ord(sid_str[3])<<32)
-            | (ord(sid_str[4])<<24)
-            | (ord(sid_str[5])<<16)
-            | (ord(sid_str[6])<<8)
-            | ord(sid_str[7]))
-    result = "S-%d-%d" % (ord(sid_str[0]), auth)
     rest = sid_str[8:]
-    for i in range(0,ord(sid_str[1])):
+    first = struct.unpack('BBBBBBBB', sid_str[0:8])
+    auth = ((first[2]<<40)
+            | (first[3]<<32)
+            | (first[4]<<24)
+            | (first[5]<<16)
+            | (first[6]<<8)
+            | first[7])
+    result = "S-%d-%d" % (first[0], auth)
+    for i in range(0,first[1]):
         if len(rest) >= 4:
             next_int = struct.unpack('<I', rest[:4])[0]
             rest = rest[4:]
@@ -80,27 +84,25 @@ def binSIDtoASCII(sid_str):
     return result
 
 
-# Returns a string which contains s, except for non-printable or
-# special characters are quoted in hex with the syntax '\xQQ' where QQ
-# is the hex ascii value of the quoted character.  specials must be a
-# sequence.
-def quoteString(s, specials="\\"):
+def quoteBinary(binary, specials=b"%"):
+    # Doing it this way to support python 2 and 3
+    b = struct.unpack('B'*len(binary), binary)
+    specials = struct.unpack('B'*len(specials), specials)
     ret_val = ''
-    for c in s:
-        o = ord(c)
-        if (o < 32) or (o > 126) or (c in specials):
-            ret_val += ("\\x%.2X" % o)
+    for c in b:
+        if (c < 32) or (c > 126) or (c in specials):
+            ret_val += ("%%%.2X" % c)
         else:
-            ret_val += c
+            ret_val += chr(c)
     
     return ret_val
 
 
-def quoteUnicode(s, specials=u'\\\r\n'):
-    ret_val = u''
+def quoteUnicode(s, specials='%\r\n'):
+    ret_val = ''
     for c in s:
         if c in specials:
-            ret_val += "\\x%.2X" % ord(c)
+            ret_val += "%%%.2X" % ord(c)
         else:
             ret_val += c
 
@@ -284,7 +286,7 @@ class evtFile:
     #    (parse_meta == 1 && found cursor)   --> self.cursor['first_off']
     #
     def __init__(self, filename, message_repository, parse_meta=1):
-        self.f = file(filename, "rb")
+        self.f = open(filename, "rb")
         self.mr = message_repository
         
         if parse_meta:
@@ -369,7 +371,7 @@ class evtFile:
     # Raises: IOError
     def guessRecordType(self):
         if not self.f:
-            raise IOError, "Log file not open."
+            raise IOError("Log file not open.")
 
         wrapped_log = False
         ret_val = 'unknown'
@@ -383,7 +385,7 @@ class evtFile:
                 if (size1 > log_fixed_size and (self.size() < cur_pos+size1)):
                     if (self.size() < cur_pos+log_fixed_size):
                         return ret_val
-                    wrapped_log = true
+                    wrapped_log = True
                     self.f.seek(header_size+(cur_pos+size1-self.size()-4))
                 else:            
                     self.f.seek(cur_pos+size1-4)
@@ -420,7 +422,7 @@ class evtFile:
         raw_rec = self.f.read(fmt_len)
         
         if len(raw_rec) < fmt_len:
-            raise EOFError, "Record read is too short for format."
+            raise EOFError("Record read is too short for format.")
 
         (size1,lfle,unknown1,unknown2,
          first_off,next_off,next_num,first_num,
@@ -451,7 +453,7 @@ class evtFile:
         raw_rec = self.f.read(fmt_len)
 
         if len(raw_rec) < fmt_len:
-            raise EOFError, "Record read is too short for format."
+            raise EOFError("Record read is too short for format.")
         
         (size1,magic1,magic2,magic3,magic4,
          first_off,next_off,next_num,first_num,
@@ -471,7 +473,7 @@ class evtFile:
     def getLogRecord(self):
         size_str = self.f.read(4)
         if len(size_str) < 4:
-            raise EOFError, "Couldn't read record length"
+            raise EOFError("Couldn't read record length")
         (size,) = struct.unpack('<I', size_str)
     
         fixed_fmt = '<IIIIHHHHHHIIIIII'
@@ -479,8 +481,7 @@ class evtFile:
         
         if size < fixed_fmt_len:
             # XXX: use different exception class here
-            raise ValueError, ("Log size (%d) is too small.  Not a log record?"
-                               % size)
+            raise ValueError("Log size (%d) is too small.  Not a log record?" % size)
 
         rec_str = self.f.read(size-4)
         if len(rec_str) < fixed_fmt_len:
@@ -510,12 +511,13 @@ class evtFile:
         # Grab template variables
         strs = []
         if string_offset > 0:
-            strs=rec_str[string_offset-4:data_offset-4].decode(
-                source_encoding,'replace').split(u'\x00')
+            strs=rec_str[string_offset-4:data_offset-4].decode(source_encoding,
+                                                               'replace').split('\x00')
             
         # Grab source and computer fields
         vstr = variable_str.decode(source_encoding,
-                                   'replace').split(u'\x00', 2)
+                                   'replace').split('\x00', 2)
+
         source = ''
         if len(vstr) > 0:
             source = vstr[0]
@@ -537,7 +539,8 @@ class evtFile:
             data = rec_str[data_offset-4:data_offset+data_len-4]
     
         # Retrieve and process message template
-        event_rva = (long(event_rva_offset) << 16) | event_id
+        event_rva = (event_rva_offset << 16) | event_id
+
         message_template = self.mr.getMessageTemplate(source, event_rva)
         message = ''
         if message_template:
@@ -562,7 +565,7 @@ class evtFile:
                 'event_id':event_id, 'event_rva':"0x%s" % event_rva,
                 'user':sid, 'computer':computer,
                 'message':message,
-                'strings':'|'.join(strs).strip('|'),
+                'strings':strs,
                 'data':data}
 
 
@@ -594,13 +597,14 @@ class messageRepository:
         msg_dir = "%s/messages" % topdir
         dbs = os.listdir(msg_dir)
         for db in dbs:
+            db = db[:len(db)-3]
             db_file = "%s/%s" % (msg_dir,db)
-            self.msg_dbs[db] = anydbm.open(db_file, "r", 0644)
+            self.msg_dbs[db] = dbm.open(db_file, "r", 0o644)
 
         log_dir = "%s/services" % topdir
         for t in ("category", "event", "parameter"):
-            db_file = "%s/%s/%s.db" % (log_dir, log, t)
-            self.svc_dbs[t] = anydbm.open(db_file, "r")
+            db_file = "%s/%s/%s" % (log_dir, log, t)
+            self.svc_dbs[t] = dbm.open(db_file, "r")
 
 
     def getMessageTemplate(self, service, rva, lang_code=None):
@@ -609,10 +613,11 @@ class messageRepository:
         else:
             langs = self.languages
         
+        service = service.lower().encode('utf-8', 'replace')
         for lang in langs:
-            mdbs = self.svc_dbs["event"].get(service.lower().encode('ascii'),
-                                             None)
+            mdbs = self.svc_dbs["event"].get(service, None)
             if mdbs:
+                mdbs = mdbs.decode('utf-8', 'replace')
                 for mdb in mdbs.split(':'):
                     ret_val = self.msg_dbs[mdb].get("%s-%.8X"%(lang, rva), None)
                     if ret_val:
@@ -649,6 +654,7 @@ class grokevtConfig:
                                                        % self.profile_path)
             self.drive_mapping = self.readMappingFromFiles("%s/drives"
                                                            % self.profile_path)
+
     
     def readMappingFromFiles(self, dir):
         ret_val = {}
@@ -657,7 +663,7 @@ class grokevtConfig:
             for k in os.listdir(dir):
                 l = self.readLineFromFile("%s/%s" % (dir, k))
                 if l != None:
-                    ret_val[k] = l
+                    ret_val[k] = l.decode('utf-8', 'replace')
         else:
             sys.stderr.write("WARNING: Could not read configuration"\
                              +" directory '%s'.\n" % dir)
@@ -669,8 +675,8 @@ class grokevtConfig:
         # XXX: the os.access() call may not work correctly in suid situations.
         if os.path.isfile(fp):
             if os.access(fp, os.R_OK):
-                f = file(fp, "r")
-                ret_val = f.readline().rstrip('\n\r')
+                f = open(fp, "rb")
+                ret_val = f.readline().rstrip(b'\n\r')
                 f.close()
             else:
                 sys.stderr.write("WARNING: Could not read configuration"\
